;+++++++++++++++++++++++++++++++++++++++++++++
; Procedure King.pro to create GC King profile
; Written by M.Paolillo on 13/1/04; modified on 7/6/04

;=== !!! This is a modified version of the king.pro IDL procedure which does ==
;=== !!! not include detector noise and sky background. It is designed to    ==
;=== !!! generate simulated GC images to be added to a real ACS exposure     ==
;=== !!! which already includes detector noise and background. 

;+++++++++++++++++++++++++++++++++++++++++++++



; ============================
;        Define King function 
;============================

; King model given King parameters, coordinates and center 
;   (Note: model is not normalized!)
function king, x, y, x0, y0, r0, c
 r = sqrt( (x-x0)^2 + (y-y0)^2 )
 if (r/r0 lt c) then $
    king = ( ( 1./sqrt(1+(r/r0)^2) ) - ( 1./sqrt(1+c^2) ) )^2 $
 else king = 0.
 return, king
end




;=======================================
; Begin procedure to create 2D King fits model
;=======================================
PRO kingmod_addacs, name, xdim, ydim, pixscale, r0, c, x0, y0, totmag,$
    	obstime, psfimg     
; - input param: output image name. Image dimensions and pix.scale (x y arcsec/pix),King parameters in image pixels (r0 c=r_t/r0), King model center in image pixels (x0 y0), King model total magnitude (in V band), observing time (sec), PSF image for convolution


; Noise parameters are pre-defined for adding to real ACS images. The two original lines are thus commented
noiseindx="yes" ; Define as string and set default to include noise
;noiseindx="no" ; Used for test purposes
expnum=1.   ;assumes single exposure to compute noise



; ---- Make image -----
osamp=10.	;oversampling factor for integration
kingarr_tmp = make_array(xdim*osamp, ydim*osamp, /double)	; create new oversampled array


; -- Transform center coordinates to account for rebinning 
;    considering half bin shift in bin coordinates (as opposed to absolute ones)

x1 = ( x0 - 0.5 ) * osamp + 0.5
y1 = ( y0 - 0.5 ) * osamp + 0.5



; -- Compute King values in the center of each pixel in oversapled array --
for i = 0, xdim*osamp-1 do begin
    for j = 0, ydim*osamp-1 do begin
    	kingarr_tmp[i,j] = king( i, j, x1-1, y1-1, r0*osamp, c ) ;Center is corrected since array start from (0,0)
	;print, kingarr_tmp[i,j] ; test line
    endfor
endfor

mwrfits, kingarr_tmp, "osamp_kingarr.fits", imghead, /create	; output oversampled array for test purposes

; --- Rebin to initial size ---
kingarr = make_array(xdim, ydim, /double)	; create new array
for i = 0, xdim-1 do begin
    for j = 0, ydim-1 do begin
    	kingarr[i,j] = total( kingarr_tmp[i*osamp:i*osamp+(osamp-1), j*osamp:j*osamp+(osamp-1)] )/osamp^2. ; rebin array
    endfor
endfor


; --- Normalize to total mag ---
totflux = 10.^( -0.4 * ( totmag + 21.12 ) )   ;Mag constant for V band

;conv_fac = 5.07e18    ; cnts/s for HST ACS, F555W for const. flux source of 1 erg/s/cm^-2/Ang
conv_fac = 1.27e19     ; cnts/s for HST ACS, F606W for const. flux source of 1 erg/s/cm^-2/Ang


king_norm = conv_fac * totflux / total( kingarr ) ; normalize output and convert in cnt/s



; --- Add background ----

; Bkg set to 0 because the bkg should come from the actual image (see header)
bkgrate = 0.
;bkgrate = conv_fac * 10.^( -0.4 * ( bkg + 21.12 ) ) * pixscale^2   ;Mag constant for V band

king_bkg_arr = (king_norm*kingarr+bkgrate) * obstime
;print, kingarr, king_bkg_arr


; --- Convolve with PSF ----
if (psfimg ne 'no') then begin
    fits_read, psfimg, psfarr
    ;king_bkg_conv_arr = convolve( king_bkg_arr, psfarr)
    king_bkg_conv_arr = convol( king_bkg_arr, psfarr, /edge_truncate)	; to use IDL procedure without 180 deg rotation
endif else king_bkg_conv_arr = king_bkg_arr
;print, king_bkg_conv_arr


; --- Add noise ---

; This section is modified with respect to the original king.pro to 
; be used on actual ACS images

if noiseindx eq 'yes' then begin
 ; Define new arrays
 king_noise_src = king_bkg_conv_arr - king_bkg_conv_arr ; Define empty array with (x,y)=0 everywhere
; king_noise_bkg = king_noise_src
; king_noise_readout = replicas( king_noise_src, expnum)
; king_noise_dark = king_noise_src
; king_noise_sky = king_noise_src

; readout = (5.)^2	 ; ACS readout signal
; dark = 2.2e-3 * obstime  ; ACS dark current
; sky = 0.0545 * obstime  ;average sky background

 for i = 0, xdim-1 do begin
     for j = 0, ydim-1 do begin
 	 seed_src = (1000.*i + 100.*j + 10.)*randomu(seed) ;src Poisson random seed, randomu function avoids repeating sequence in future calls
 	 src_mean_cnts = king_bkg_conv_arr[i,j] - bkgrate * obstime  ;average counts from source
 	 ;print, king_bkg_conv_arr[i,j], bkgrate * obstime, src_mean_cnts
 	 king_noise_src[i,j] = randomn(seed_src, poisson = src_mean_cnts+1.e-3) ;counts from src (constant to avoid 0 mean)
 	 ;seed_bkg = 1000.*i + 100.*j + 20. ;bkg Poisson random seed
 	 ;king_noise_bkg[i,j] = randomn(seed_bkg, poisson = bkgrate * obstime)  ;counts from bkg
 	 ;for l=0,expnum-1 do begin
 	 ;    seed_readout = 1000.*i + 100.*j + 30. + l ;readout Poisson random seed
 	 ;    king_noise_readout[i,j,l] = randomn(seed_readout, poisson = readout)  ;counts from readout
 	 ;endfor
 	 ;seed_dark = 1000.*i + 100.*j + 40. ;bkg Poisson random seed
 	 ;king_noise_dark[i,j] = randomn(seed_dark, poisson = dark)  ;counts from bkg
 	 ;seed_sky = 1000.*i + 100.*j + 50. ;sky Poisson random seed
 	 ;king_noise_sky[i,j] = randomn(seed_sky, poisson = sky)  ;counts from sky
 	 ;print, king_noise_src[i,j], king_noise_bkg[i,j], (total(king_noise_readout,3))[i,j], king_noise_dark[i,j], king_noise_sky[i,j]
 	 ;if i eq 50 and j eq 50 then stop
     endfor
 endfor

king_bkg_conv_arr = king_noise_src	 ; noisy signals
; king_bkg_conv_arr = king_noise_src + king_noise_bkg + total(king_noise_readout,3) + king_noise_dark + king_noise_sky	 ; add up all signals

endif

; ---- Output image -----
imghead=make_array(12,/string)	; create header
imghead[0]="PSF convolved King model to be added to ACS images"
imghead[1]="i.e. no bkg, sky or detector noise included"
imghead[2]="Model created with following parameters:"
imghead[3]="Core radius="+string(r0)+" pixels"
imghead[4]="Concentration index (r_t/r_0)="+string(c)
imghead[5]="Center in image pixels (x0 y0)="+string(x0)+string(y0)
imghead[6]="Total magnitude (within the image)="+string(totmag)
imghead[7]="Count rate conversion factor in V="+string(conv_fac)+" in cnt/s per erg/s/cm^-2/Ang"
imghead[8]="Observing time (sec)="+string(obstime)
imghead[9]="Convolved with PSF: "+string(psfimg)
imghead[10]="Src noise: "+string(noiseindx)




mwrfits, king_bkg_conv_arr, name, imghead, /create
print, "Wrote image: "+name

;close, /all	; don't need this if called within other procedures
end





